package com.zsc.edu.gateway.modules.iot.device.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.zsc.edu.gateway.modules.iot.device.dto.BatchDeviceDto;
import com.zsc.edu.gateway.modules.iot.device.dto.DeviceDto;
import com.zsc.edu.gateway.modules.iot.device.dto.DeviceServeDto;
import com.zsc.edu.gateway.modules.iot.device.entity.Device;
import com.zsc.edu.gateway.modules.iot.device.query.DeviceQuery;
import com.zsc.edu.gateway.modules.iot.device.service.DeviceService;
import com.zsc.edu.gateway.modules.iot.device.vo.DeviceVo;
import com.zsc.edu.gateway.modules.iot.record.entity.Record;
import com.zsc.edu.gateway.modules.iot.record.service.RecordService;
import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author zhuang
 */
@AllArgsConstructor
@RestController
@RequestMapping("api/rest/device")
public class DeviceController {

    DeviceService service;

    RecordService recordService;

    /**
     * 创建设备
     */
    @PostMapping
    public Device create(@RequestBody DeviceDto deviceDto) {
        return service.create(deviceDto);
    }


    /**
     * 批量创建设备
     */
    @PostMapping("batch")
    public List<Device> batchCreate(@RequestBody BatchDeviceDto batchDeviceDto) {
        return service.batchCreate(batchDeviceDto);
    }


    /**
     * 更新设备
     */
    @PatchMapping("{id}")
    public Device update(@RequestBody DeviceDto deviceDto, @PathVariable("id") Long id) {
        return service.update(deviceDto, id);
    }


    /**
     * 查询设备
     */
    //TODO IPage的分页条数问题
    @GetMapping
    public IPage<DeviceVo> query(Page<DeviceVo> page, DeviceQuery query) {
        return service.query(page, query);
    }

    /**
     * 删除设备
     */
    @DeleteMapping("{id}")
    public Boolean delete(@PathVariable("id") Long id) {
        return service.removeById(id);
    }

    /**
     * 设备
     * 查询详情
     */
    @GetMapping("{id}")
    public DeviceVo detail(@PathVariable("id") Long id) {
        return service.detail(id);
    }


    /**
     * 手动触发事件
     */
    @PostMapping("serve")
    public ResponseEntity<String> serve(@RequestBody DeviceServeDto dto) {
        if (service.serve(dto)) {
            return ResponseEntity.ok("服务执行成功");
        } else {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("服务执行失败");
        }
    }

//    /**
//     * 查询设备上报记录
//     *
//     * @param clientId 设备的查询表单
//     * @param page 分页参数
//     * @return Page<Device> 设备分页数据
//     */
//    @GetMapping("record")
//    public Page<Record> pageRecord(String clientId,Page<Record> page) Pageable pageable) {
//        return recordService.page(clientId, pageable);
//    }

    /**
     * 查询设备上报记录
     */
    @GetMapping("record/photo")
    public List<Record> recordPhoto(String clientId) {
        return recordService.recordPhoto(clientId);
    }

}
