package com.zsc.edu.gateway.modules.iot.record.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.framework.redis.RedisUtils;
import com.zsc.edu.gateway.modules.iot.record.entity.Record;
import com.zsc.edu.gateway.modules.iot.record.repo.RecordRepository;
import com.zsc.edu.gateway.modules.iot.record.service.RecordService;
import jakarta.annotation.Resource;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

/**
 * @author zhuang
 */
@AllArgsConstructor
@Service
public class RecordServiceImpl extends ServiceImpl<RecordRepository, Record> implements RecordService {
    RedisUtils redisUtils;

    @Override
    public Record create(Record record) {
        save(record);
        return record;
    }

    @Override
    public List<Record> page(IPage<Record> page, String clientId) {
        LambdaQueryWrapper<Record> queryWrapper = new LambdaQueryWrapper<>();
        if (clientId != null) {
            queryWrapper.eq(Record::getClientId, clientId);
            return baseMapper.selectList(page, queryWrapper);
        }
        return baseMapper.selectList(page, queryWrapper);
    }

    @Override
    public List<Record> recordPhoto(String clientId) {
        LocalDateTime recordTime = LocalDateTime.parse(redisUtils.get("serve:sendTime:photograph:" + clientId));
        List<Record> records = baseMapper.findAllByClientIdAndRecordTimeAfter(clientId, recordTime);
        Optional<Record> first = records.stream().max(Comparator.comparing(Record::getRecordTime));
        if (first.isPresent()) {
            String recordTimeStr = (String) first.get().getContent().get("recordTimeStr");
            return baseMapper.findByRecordTimeStr(recordTimeStr);
        } else {
            return records;
        }

    }
}
