package com.zsc.edu.gateway.modules.iot.tsl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.exception.ConstraintException;
import com.zsc.edu.gateway.modules.iot.tsl.dto.EventDto;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ParamDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Event;
import com.zsc.edu.gateway.modules.iot.tsl.entity.EventParam;
import com.zsc.edu.gateway.modules.iot.tsl.entity.Param;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.EventMapper;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ParamMapper;
import com.zsc.edu.gateway.modules.iot.tsl.query.EventQuery;
import com.zsc.edu.gateway.modules.iot.tsl.repo.EventParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.EventRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.service.EventService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Yao
 */
@Service
@AllArgsConstructor
public class EventServiceImpl extends ServiceImpl<EventRepository, Event> implements EventService {

    private final EventMapper mapper;
    private final ParamMapper paramMapper;
    private final ParamRepository paramRepo;
    private final EventParamRepository eventParamRepo;

    /**
     * 新建物模型事件
     */
    @Override
    public Event create(EventDto dto) {
        if (baseMapper.findByName(dto.getName()) != null) {
            throw new ConstraintException("该事件已存在！");
        }
        Event event = new Event();
        event.setName(dto.getName());
        event.setIdentifier(dto.getIdentifier());
        event.setType(dto.getType());
        event.setProductId(dto.getProductId());
        save(event);
        List<Long> eventIds = new ArrayList<>();
        if (dto.getOutputs() != null) {
            dto.getOutputs().forEach(paramDto -> {
                Param param = paramMapper.toEntity(paramDto);
                paramRepo.insert(param);
                eventIds.add(param.getId());
            });
        }
        List<EventParam> eventParams = eventIds.stream()
                .map(eventId -> new EventParam(event.getId(), eventId))
                .toList();
        eventParamRepo.insert(eventParams);
        return event;
    }
    /**
     * 更新物模型事件
     */
    @Override
    public Event update(EventDto dto, Long id) {
        this.lambdaUpdate().eq(Event::getId, id)
                .set(Event::getName, dto.getName())
                .set(Event::getIdentifier, dto.getIdentifier())
                .set(Event::getType, dto.getType())
                .set(Event::getProductId, dto.getProductId())
                .update();
        LambdaQueryWrapper<EventParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(EventParam::getEventId, id);
        List<EventParam> eventParams = eventParamRepo.selectList(wrapper);
        List<Long> paramIds = eventParams.stream()
                .map(EventParam::getParamId)
                .toList();
        List<Param> params = paramRepo.selectByIds(paramIds);
        if (dto.getOutputs() != null && !dto.getOutputs().isEmpty()) {
            for (int i = 0; i < dto.getOutputs().size(); i++) {
                ParamDto paramDto = dto.getOutputs().get(i);
                Param param = params.get(i);
                paramMapper.convert(paramDto, param);
                paramRepo.updateById(param);
            }
        }
        return this.getById(id);
    }
    /**
     * 分页查询物模型事件
     *
     * @param query 查询表单
     * @param page  分页参数
     */
    @Override
    public IPage<Event> page(Page<Event> page, EventQuery query) {
        return baseMapper.page(page, query);
    }

    /**
     * 查询详情
     *
     * @param id Id
     * @return Event
     */
    @Override
    public Event detail(Long id) {
        return baseMapper.selectById(id);
    }

    /**
     * 删除
     */
    @Override
    public boolean delete(Long id) {
        removeById(id);
        LambdaQueryWrapper<EventParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(EventParam::getEventId, id);
        List<EventParam> eventParams = eventParamRepo.selectList(wrapper);
        if (!eventParams.isEmpty()) {
            List<Long> paramIds = eventParams.stream()
                    .map(EventParam::getParamId)
                    .toList();
            paramRepo.deleteByIds(paramIds);
        }
        eventParamRepo.delete(wrapper);
        return true;
    }
}
