package com.zsc.edu.gateway.modules.iot.tsl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.gateway.exception.ConstraintException;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ParamDto;
import com.zsc.edu.gateway.modules.iot.tsl.dto.ServeDto;
import com.zsc.edu.gateway.modules.iot.tsl.entity.*;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ParamMapper;
import com.zsc.edu.gateway.modules.iot.tsl.mapper.ServeMapper;
import com.zsc.edu.gateway.modules.iot.tsl.query.ServeQuery;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ServeParamRepository;
import com.zsc.edu.gateway.modules.iot.tsl.repo.ServeRepository;
import com.zsc.edu.gateway.modules.iot.tsl.service.ServeService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;


/**
 * @author 15864
 */
@AllArgsConstructor
@Service
public class ServeServiceImpl extends ServiceImpl<ServeRepository, Serve> implements ServeService {

    private final ServeMapper mapper;
    private final ParamMapper paramMapper;
    private final ParamRepository paramRepository;
    private final ServeParamRepository serveParamRepository;
    /**
     * 新建功能
     */
    @Override
    public Serve create(ServeDto dto) {
        if (baseMapper.findByName(dto.getName()) != null) {
            throw new ConstraintException("该事件已存在！");
        }
        Serve serve = new Serve();
        serve.setProductId(dto.getProductId());
        serve.setName(dto.getName());
        serve.setIdentifier(dto.getIdentifier());
        save(serve);
        List<Long> serveIds = new ArrayList<>();
        if (dto.getInputs() != null) {
            dto.getInputs().forEach(paramDto -> {
                Param param = paramMapper.toEntity(paramDto);
                paramRepository.insert(param);
                serveIds.add(param.getId());
            });
        }
        if (dto.getOutputs() != null) {
            dto.getOutputs().forEach(paramDto -> {
                Param param = paramMapper.toEntity(paramDto);
                paramRepository.insert(param);
                serveIds.add(param.getId());
            });
        }
        List<ServeParam> serveParams = serveIds.stream()
                .map(serveId -> new ServeParam(serve.getId(), serveId))
                .toList();
        serveParamRepository.insert(serveParams);
        return serve;
    }
    /**
     * 更新功能
     */
    @Override
    public Serve update(ServeDto dto, Long id) {
        this.lambdaUpdate().eq(Serve::getId, id)
                .set(Serve::getName, dto.getName())
                .set(Serve::getIdentifier, dto.getIdentifier())
                .set(Serve::getProductId, dto.getProductId())
                .update();

        LambdaQueryWrapper<ServeParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ServeParam::getServeId, id);
        List<ServeParam> serveParams = serveParamRepository.selectList(wrapper);
        List<Long> paramIds = serveParams.stream()
                .map(ServeParam::getParamId)
                .toList();
        List<Param> params = paramRepository.selectByIds(paramIds);
        List<ParamDto> allParamsDto = new ArrayList<>();
        if (dto.getInputs() != null && !dto.getInputs().isEmpty()) {
            allParamsDto.addAll(dto.getInputs());
        }
        if (dto.getOutputs() != null && !dto.getOutputs().isEmpty()) {
            allParamsDto.addAll(dto.getOutputs());
        }

        for (int i = 0; i < allParamsDto.size(); i++) {
            ParamDto paramDto = allParamsDto.get(i);
            Param param = params.get(i);
            paramMapper.convert(paramDto, param);
            paramRepository.updateById(param);
        }

        return this.getById(id);
    }

    /**
     * 分页查询功能
     *
     * @param query 查询表单
     * @param page  分页参数
     */
    @Override
    public IPage<Serve> page(Page<Serve> page, ServeQuery query) {
        return baseMapper.page(page, query);
    }

    /**
     * 查询详情
     *
     * @param id 主键
     * @return Serve
     */
    @Override
    public Serve detail(Long id) {
        return baseMapper.selectById(id);
    }

    @Override
    public Boolean delete(Long id) {
        removeById(id);
        LambdaQueryWrapper<ServeParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ServeParam::getServeId, id);
        List<ServeParam> serveParam = serveParamRepository.selectList(wrapper);
        if (!serveParam.isEmpty()) {
            List<Long> paramIds = serveParam.stream()
                    .map(ServeParam::getParamId)
                    .toList();
            paramRepository.deleteByIds(paramIds);
        }
        serveParamRepository.delete(wrapper);
        return true;
    }
}
